import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { json, scaleLinear, scaleTime, extent, max, select } from 'd3';

const jsonUrl = 'https://raw.githubusercontent.com/freeCodeCamp/ProjectReferenceData/master/GDP-data.json';

const width = 900;
const height = 450;
const margin = { top: 40, bottom: 65, left: 60, right: 160 };

const setTooltip = () => {
  
}

const App = () => {
	const [data, setData] = useState(null);
  
  useEffect(() => {
  	json(jsonUrl).then(data => data["data"]).then(setData);
  }, []);
  
  if (!data) {
    return <div>Loading...</div>;
  }
  
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  const xScale = scaleTime().domain(extent(data, d => new Date(d[0]))).range([0, innerWidth]);
  const yScale = scaleLinear().domain([0, max(data, d => d[1])]).range([innerHeight, 0]);
  
  return (
  	<>
      <div id="title">United States GDP</div>
      <svg width={width} height={height}>
        <g transform={`translate(${margin.left},${margin.top})`}>
          {
          	yScale.ticks().map(tickValue => <g className='tick' transform={`translate(0,${yScale(tickValue)})`}>
                                 <line 
                                   x1={-5}
                                   stroke='black'
                                   />
                                 <text
                                   style={{textAnchor: 'end'}}
                                   dy="0.32em"
                                   x={-5}
                                   >{tickValue}</text>
                               </g>)
          }
          {
            xScale.ticks().map(tickValue => <g className='tick' transform={`translate(${xScale(tickValue)},${innerHeight})`}>
                                 <line 
                                   y2={4}
                                   stroke='black'
                                   /> 
                                 <text
                                    dy="0.71em"
                                    style={{textAnchor: 'middle'}}
                                    y={5}
                                    >{tickValue.getFullYear()}</text>
                                </g>)
          }
          <g id='y-axis' transform={`translate(0,${innerHeight})`}>
          <line 
            x2={innerWidth}
            stroke='black'
            />
          </g>
          <g id='x-axis'>
          <line 
            y2={innerHeight}
            stroke='black'
            />
          </g>
          <text
            transform={`translate(20,${innerHeight * 3 / 4}) rotate(-90)`}
            id='axis-label'
            style={{anchorText: 'middle'}}
            >Gross Domestic Product</text>
          {
          	data.map(d => { 
              const date = new Date(d[0]);
              const data_date = date.getFullYear() + ' Q' + (Math.floor((date.getMonth() + 1)/3+1)); 
              const data_gdp = "$" + d[1] + ' Billion';
              
              return (<g 
                        className='bars' 
                        transform={`translate(${xScale(new Date(d[0]))}, ${yScale(d[1])})`}
                        onMouseEnter={() => {
                  	select('#tooltip').append('span').text(data_date);
                    select('#tooltip').append('span').text(data_gdp);
                    select('#tooltip').style('left', xScale(new Date(d[0])) + 80 + 'px');
                    select('#tooltip').style('top', innerHeight - 40 + 'px');
                  	select('#tooltip').style('visibility','visible');
                  }}
                        onMouseLeave={() => {
                  	select('#tooltip').selectAll('span').remove();
                    select('#tooltip').style('visibility','hidden');
                  }}
                        >
                     <rect 
                       height={innerHeight - yScale(d[1])}
                       width={innerWidth/data.length}
                       />
                     </g>);}) 
          }
        </g>
      </svg>
      <div id='tooltip'></div>
    </>
  );

}

ReactDOM.render(<App />, document.getElementById('root'));
